# -*- coding: utf-8 -*-
#
# Copyright (C) 2016-2023 by the Free Software Foundation, Inc.
#
# This file is part of Django-Mailman.
#
# Django-Mailman is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Django-Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Django-Mailman.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.models import User
from django.test import TestCase

from django_mailman3.forms import UserProfileForm


class TestUserProfileForm(TestCase):

    def setUp(self):
        self.user = User.objects.create(username='bob',
                                        email='bob@example.com')

    def tearDown(self):
        self.user.delete()

    def test_valid_form(self):
        data = dict(username='alice',
                    first_name='Alice',
                    last_name='Wonderland',
                    timezone='UTC')
        form = UserProfileForm(data=data)
        self.assertTrue(form.is_valid())

    def test_invalid_form(self):
        data = dict(username='alice',
                    first_name='Alice',
                    last_name='',
                    timezone='UTC')
        form = UserProfileForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertTrue('This field is required.' in str(form.errors))
        self.assertTrue('last_name' in str(form.errors))

    def test_form_cleans_username(self):
        data = dict(username='bob',
                    first_name='Bob',
                    last_name='User',
                    timezone='UTC')
        form = UserProfileForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertTrue('A user with that username already exists.'
                        in str(form.errors))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016-2023 by the Free Software Foundation, Inc.
#
# This file is part of Django-Mailman.
#
# Django-Mailman is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Django-Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Django-Mailman.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aurelien Bompard <abompard@fedoraproject.org>
#


from django import forms
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _

from django_mailman3.models import TIMEZONES


class UserProfileForm(forms.Form):
    username = forms.CharField(required=True, label=_('Username'))
    first_name = forms.CharField(label=_('First name'))
    last_name = forms.CharField(label=_('Last name'))
    timezone = forms.ChoiceField(
        label=_('Time zone'), choices=TIMEZONES)

    def clean_username(self):
        username = self.cleaned_data.get("username")
        if username != self.initial.get("username"):
            if User.objects.filter(username=username).exists():
                raise forms.ValidationError(
                    _("A user with that username already exists."))
        return username



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016-2023 by the Free Software Foundation, Inc.
#
# This file is part of Django-Mailman.
#
# Django-Mailman is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Django-Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Django-Mailman.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aurelien Bompard <abompard@fedoraproject.org>
#


from django.template import Context
from django.test import RequestFactory, SimpleTestCase

from django_mailman3.lib.paginator import MailmanPaginator, paginate
from django_mailman3.templatetags.pagination import (
    add_to_query_string, paginator)
from django_mailman3.tests.utils import FakeMMPage, TestCase


class TestMailmanPaginator(TestCase):

    def setUp(self):
        super(TestMailmanPaginator, self)._pre_setup()
        # Create a user pagination based on just list of numbers instead of
        # list of users. This is a mocked page and thus doesn't really care.
        self.mailman_client.get_user_page.return_value = FakeMMPage(
            count=5, entries=list(range(100)))
        self.user_pages = MailmanPaginator(
            function=self.mailman_client.get_user_page, per_page=5)

    def test_get_page_one(self):
        user_page_1 = self.user_pages.page(1)
        self.assertIsNotNone(user_page_1)
        # The number of entries in this list should be 5.
        self.assertEqual(len(list(user_page_1)), 5)

    def test_get_count(self):
        # Total number of elements in this paginator class should be 100
        self.assertEqual(self.user_pages.count, 100)


class PaginateTestCase(SimpleTestCase):

    def test_page_range_ellipsis(self):
        objects = list(range(1000))
        self.assertEqual(
            paginate(objects, 1, 20).paginator.page_range_ellipsis,
            [1, 2, 3, 4, '...', 50])
        self.assertEqual(
            paginate(objects, 2, 20).paginator.page_range_ellipsis,
            [1, 2, 3, 4, 5, '...', 50])
        self.assertEqual(
            paginate(objects, 3, 20).paginator.page_range_ellipsis,
            [1, 2, 3, 4, 5, 6, '...', 50])
        self.assertEqual(
            paginate(objects, 4, 20).paginator.page_range_ellipsis,
            [1, 2, 3, 4, 5, 6, 7, '...', 50])
        self.assertEqual(
            paginate(objects, 5, 20).paginator.page_range_ellipsis,
            [1, 2, 3, 4, 5, 6, 7, 8, '...', 50])
        self.assertEqual(
            paginate(objects, 6, 20).paginator.page_range_ellipsis,
            [1, 2, 3, 4, 5, 6, 7, 8, 9, '...', 50])
        self.assertEqual(
            paginate(objects, 7, 20).paginator.page_range_ellipsis,
            [1, '...', 4, 5, 6, 7, 8, 9, 10, '...', 50])
        self.assertEqual(
            paginate(objects, 8, 20).paginator.page_range_ellipsis,
            [1, '...', 5, 6, 7, 8, 9, 10, 11, '...', 50])
        self.assertEqual(
            paginate(objects, 9, 20).paginator.page_range_ellipsis,
            [1, '...', 6, 7, 8, 9, 10, 11, 12, '...', 50])
        self.assertEqual(
            paginate(objects, 10, 20).paginator.page_range_ellipsis,
            [1, '...', 7, 8, 9, 10, 11, 12, 13, '...', 50])
        self.assertEqual(
            paginate(objects, 30, 20).paginator.page_range_ellipsis,
            [1, '...', 27, 28, 29, 30, 31, 32, 33, '...', 50])
        self.assertEqual(
            paginate(objects, 40, 20).paginator.page_range_ellipsis,
            [1, '...', 37, 38, 39, 40, 41, 42, 43, '...', 50])
        self.assertEqual(
            paginate(objects, 41, 20).paginator.page_range_ellipsis,
            [1, '...', 38, 39, 40, 41, 42, 43, 44, '...', 50])
        self.assertEqual(
            paginate(objects, 42, 20).paginator.page_range_ellipsis,
            [1, '...', 39, 40, 41, 42, 43, 44, 45, '...', 50])
        self.assertEqual(
            paginate(objects, 43, 20).paginator.page_range_ellipsis,
            [1, '...', 40, 41, 42, 43, 44, 45, 46, '...', 50])
        self.assertEqual(
            paginate(objects, 44, 20).paginator.page_range_ellipsis,
            [1, '...', 41, 42, 43, 44, 45, 46, 47, '...', 50])
        self.assertEqual(
            paginate(objects, 45, 20).paginator.page_range_ellipsis,
            [1, '...', 42, 43, 44, 45, 46, 47, 48, 49, 50])
        self.assertEqual(
            paginate(objects, 46, 20).paginator.page_range_ellipsis,
            [1, '...', 43, 44, 45, 46, 47, 48, 49, 50])
        self.assertEqual(
            paginate(objects, 47, 20).paginator.page_range_ellipsis,
            [1, '...', 44, 45, 46, 47, 48, 49, 50])
        self.assertEqual(
            paginate(objects, 48, 20).paginator.page_range_ellipsis,
            [1, '...', 45, 46, 47, 48, 49, 50])
        self.assertEqual(
            paginate(objects, 49, 20).paginator.page_range_ellipsis,
            [1, '...', 46, 47, 48, 49, 50])
        self.assertEqual(
            paginate(objects, 50, 20).paginator.page_range_ellipsis,
            [1, '...', 47, 48, 49, 50])

    def test_default_page(self):
        self.assertEqual(paginate(list(range(100)), None).number, 1)

    def test_last_page(self):
        self.assertEqual(paginate(list(range(100)), 1000).number, 10)

    def test_page_str(self):
        try:
            self.assertEqual(paginate(list(range(1000)), "2").number, 2)
        except TypeError as e:
            self.fail(e)

    def test_page_not_an_int(self):
        self.assertEqual(paginate(list(range(100)), "dummy").number, 1)

    def test_add_to_query_string(self):
        request = RequestFactory().get("/url", {"key1": "value1"})
        result = add_to_query_string(
            {"request": request}, "key2", "value2", key3="value3")
        self.assertEqual(
            set(result.split("&amp;")),
            set(["key1=value1", "key2=value2", "key3=value3"]))

    def test_paginator_tag(self):
        objects = paginate(list(range(100)), 1, 20)
        context = Context({'title': 'My Title'})
        updated_context = paginator(context, objects)
        self.assertEqual(updated_context['label_previous'], 'Previous')
        self.assertEqual(updated_context['label_next'], 'Next')
        self.assertEqual(updated_context['page'], objects)
        self.assertEqual(updated_context['per_page_options'],
                         [10, 25, 50, 100, 200])
        dated_pages_ctxt = paginator(context, objects, bydate=True)
        self.assertEqual(dated_pages_ctxt['label_previous'], 'Newer')
        self.assertEqual(dated_pages_ctxt['label_next'], 'Older')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016-2023 by the Free Software Foundation, Inc.
#
# This file is part of Django-Mailman.
#
# Django-Mailman is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Django-Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Django-Mailman.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aurelien Bompard <abompard@fedoraproject.org>
#


import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.cache import cache
from django.db.models.signals import post_save
from django.dispatch import Signal, receiver

from allauth.account.models import EmailAddress
from allauth.account.signals import (
    email_changed, email_confirmed, email_removed,
    user_logged_in, user_signed_up)
from allauth.socialaccount.signals import social_account_added

from django_mailman3.lib.mailman import (
    add_address_to_mailman_user, get_mailman_user, get_subscriptions,
    sync_email_addresses, update_preferred_address)
from django_mailman3.models import Profile


logger = logging.getLogger(__name__)

# Fields for settings.AUTH_USER_MODEL. These are for the default Django's
# contrib.auth.User model object.
FIELD_FIRST_NAME = 'first_name'
FIELD_LAST_NAME = 'last_name'

#
# Defined signals
#
domain_created = Signal()
domain_deleted = Signal()
mailinglist_created = Signal()
mailinglist_modified = Signal()
mailinglist_deleted = Signal()
user_subscribed = Signal()
user_unsubscribed = Signal()

#
# Signals listened to
#


@receiver([user_subscribed, user_unsubscribed])
def on_user_subscribed(sender, **kwargs):
    """Clear the subscriptions cache when a user is subscribed or unsubscribed.
    """
    user_email = kwargs["user_email"]
    User = get_user_model()
    try:
        user = User.objects.get(email=user_email)
    except User.DoesNotExist:
        try:
            emailaddress = EmailAddress.objects.get(email=user_email)
        except EmailAddress.DoesNotExist:
            return  # No such user
        user = emailaddress.user
    cache.delete("User:%s:subscriptions" % user.id, version=2)
    get_subscriptions(user)


@receiver(post_save, sender=settings.AUTH_USER_MODEL)
def create_profile(sender, **kwargs):
    """Create a Profile when a User is created"""
    user = kwargs["instance"]
    if not Profile.objects.filter(user=user).exists():
        Profile.objects.create(user=user)
    # If the current user's first or last name is not None, then update them in
    # Core.
    if user.first_name or user.last_name:
        mm_user = get_mailman_user(user)
        new_display_name = "{} {}".format(user.first_name, user.last_name)
        # Remove leading or trailing space if only first_ or last_.
        new_display_name = new_display_name.strip()
        if mm_user and mm_user.display_name != new_display_name:
            mm_user.display_name = new_display_name
            mm_user.save()
            # Also, update the names in the address objects.
            for address in mm_user.addresses:
                if address.display_name != new_display_name:
                    address.display_name = new_display_name
                    address.save()

# Allauth


@receiver(user_logged_in)
def on_user_logged_in(sender, **kwargs):
    """Sync a user's address with Core when they logs in."""
    user = kwargs["user"]
    if not Profile.objects.filter(user=user).exists():
        Profile.objects.create(user=user)
    sync_email_addresses(user)


@receiver(user_signed_up)
def on_user_signed_up(sender, **kwargs):
    # Sent when a user signs up for an account.
    user = kwargs["user"]
    if not Profile.objects.filter(user=user).exists():
        Profile.objects.create(user=user)
    # We want to add the user to Mailman with all its verified email addresses.
    for address in EmailAddress.objects.filter(user=user):
        if address.verified:
            logger.debug("Adding email address %s to user %s",
                         address.email, user.username)
            add_address_to_mailman_user(user, address.email)


@receiver(email_removed)
def on_email_removed(sender, **kwargs):
    # Sent when an email address has been removed.
    # Remove it from Mailman.
    user = kwargs["user"]
    email_address = kwargs["email_address"]
    mm_user = get_mailman_user(user)
    if mm_user is None:
        return  # Could not get the user from Mailman.
    try:
        mm_user.addresses.remove(email_address.email)
    except ValueError:
        pass  # not in Mailman's addresses


@receiver(email_confirmed)
def on_email_confirmed(sender, **kwargs):
    # Sent after the email address in the db was updated and set to confirmed.
    # Associate it with the user and set it to verified in Mailman.
    email_address = kwargs["email_address"]
    logger.debug("Confirmed email %s of user %s",
                 email_address.email, email_address.user.username)
    add_address_to_mailman_user(email_address.user, email_address.email)


@receiver(email_changed)
def on_email_changed(sender, **kwargs):
    # Sent when a primary email address has been changed.
    user = kwargs.get('user')
    to_email_address = kwargs.get('to_email_address')
    if to_email_address is None:
        return
    update_preferred_address(user, to_email_address)


@receiver(social_account_added)
def on_social_account_added(sender, **kwargs):
    # Sent after a user connects a social account to a their local account.
    # Add to mailman emails that were marked as verified by the social account
    # provider.
    sociallogin = kwargs["sociallogin"]
    logger.debug("Social account %s added for user %s",
                 sociallogin.account, sociallogin.user.username)
    for address in sociallogin.email_addresses:
        if EmailAddress.objects.filter(email=address.email).exists():
            # TODO: should we do something if it belongs so another user?
            continue
        logger.debug("Adding email address %s to user %s",
                     address.email, sociallogin.user.username)
        address.user = sociallogin.user
        address.primary = False  # There already is a primary address.
        address.save()
        if address.verified:
            add_address_to_mailman_user(sociallogin.user, address.email)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016-2023 by the Free Software Foundation, Inc.
#
# This file is part of Django-Mailman.
#
# Django-Mailman is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Django-Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Django-Mailman.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aurelien Bompard <abompard@fedoraproject.org>
#


from unittest.mock import Mock, call, patch

from django.contrib.auth.models import User

from allauth.account.models import EmailAddress
from allauth.account.signals import (
    email_confirmed, email_removed, user_logged_in, user_signed_up)
from allauth.socialaccount.models import SocialLogin
from allauth.socialaccount.signals import social_account_added

from django_mailman3.models import Profile
from django_mailman3.signals import user_subscribed, user_unsubscribed
from django_mailman3.tests.utils import TestCase


class SignalsTestCase(TestCase):

    def setUp(self):
        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'testPass')

    def test_user_logged_in(self):
        Profile.objects.get(user=self.user).delete()
        with patch('django_mailman3.signals.sync_email_addresses') as sea:
            user_logged_in.send(sender=User, user=self.user)
        self.assertEqual(sea.call_args_list, [call(self.user)])
        self.assertTrue(Profile.objects.filter(user=self.user).exists())

    def test_user_signed_up(self):
        Profile.objects.get(user=self.user).delete()
        EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True)
        EmailAddress.objects.create(
            user=self.user, email='another@example.com', verified=False)
        with patch('django_mailman3.signals.add_address_to_mailman_user') \
                as aatmu:
            user_signed_up.send(sender=User, user=self.user)
        self.assertEqual(
            aatmu.call_args_list, [call(self.user, self.user.email)])
        self.assertTrue(Profile.objects.filter(user=self.user).exists())

    def test_email_removed(self):
        address = EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True)
        mm_user = Mock()
        with patch('django_mailman3.signals.get_mailman_user') as gmu:
            gmu.return_value = mm_user
            email_removed.send(
                sender=User, user=self.user, email_address=address)
        self.assertEqual(mm_user.addresses.remove.call_args_list,
                         [call(self.user.email)])

    def test_email_removed_no_mailman_user(self):
        address = EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True)
        with patch('django_mailman3.signals.get_mailman_user') as gmu:
            gmu.return_value = None
            email_removed.send(
                sender=User, user=self.user, email_address=address)

    def test_email_confirmed(self):
        address = EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True)
        with patch('django_mailman3.signals.add_address_to_mailman_user') \
                as aatmu:
            email_confirmed.send(sender=EmailAddress, email_address=address)
        self.assertEqual(
            aatmu.call_args_list, [call(self.user, self.user.email)])

    def test_social_account_added(self):
        verified = EmailAddress(
            email='verified@example.com', verified=True)
        unverified = EmailAddress(
            email='unverified@example.com', verified=False)
        sociallogin = SocialLogin(
            user=self.user, email_addresses=[verified, unverified])
        with patch('django_mailman3.signals.add_address_to_mailman_user') \
                as aatmu:
            social_account_added.send(sender=User, sociallogin=sociallogin)
        self.assertEqual(
            aatmu.call_args_list, [call(self.user, 'verified@example.com')])
        self.assertEqual(
            [e.user for e in EmailAddress.objects.all()],
            [self.user, self.user])

    def test_user_subscribed(self):
        with patch('django_mailman3.signals.get_subscriptions') as gs:
            user_subscribed.send(
                sender="Postorius", user_email="test@example.com")
            self.assertEqual(gs.call_count, 1)
            user_unsubscribed.send(
                sender="Postorius", user_email="test@example.com")
            self.assertEqual(gs.call_count, 2)

    def test_user_subscribed_secondary_address(self):
        EmailAddress.objects.create(
            user=self.user, email="test-2@example.com", verified=True)
        with patch('django_mailman3.signals.get_subscriptions') as gs:
            user_subscribed.send(
                sender="Postorius", user_email="test-2@example.com")
            self.assertEqual(gs.call_count, 1)
            user_unsubscribed.send(
                sender="Postorius", user_email="test-2@example.com")
            self.assertEqual(gs.call_count, 2)

    def test_create_profile(self):
        # Test that when a user is saved, we create a Profile if one doesn't
        # exist for the User. Also, we update the display name for the user and
        # their all addresses.
        mm_user = Mock()
        mm_user.display_name = None
        # Mock user has two addresses.
        mm_user.addresses = [Mock(), Mock()]

        with patch('django_mailman3.signals.get_mailman_user') as gmu:
            gmu.return_value = mm_user
            user = User.objects.create_user(
                'myuser', 'testing@example.com', 'testPass')
            # Initially, the name of user is null, so we don't update the
            # display_name in Core.
            gmu.assert_not_called()
            # Now lets update the name of the user and see if it is reflected.
            user.first_name = 'Anne'
            user.last_name = 'Person'
            user.save()
            # Assert that the mock user was fetched and that the display name
            # was set.
            gmu.assert_called_once_with(user)
            mm_user.save.assert_called_once()
            self.assertEqual(mm_user.display_name, 'Anne Person')
            for addr in mm_user.addresses:
                self.assertEqual(addr.display_name, 'Anne Person')
                addr.save.assert_called_once()

    def test_create_profile_one_name(self):
        # Same as test_create_profile but tests that first_name only has no
        # trailing space.
        mm_user = Mock()
        mm_user.display_name = None
        # Mock user has two addresses.
        mm_user.addresses = [Mock(), Mock()]

        with patch('django_mailman3.signals.get_mailman_user') as gmu:
            gmu.return_value = mm_user
            user = User.objects.create_user(
                'myuser', 'testing@example.com', 'testPass')
            # Initially, the name of user is null, so we don't update the
            # display_name in Core.
            gmu.assert_not_called()
            # Now lets update the name of the user and see if it is reflected.
            user.first_name = 'Anne'
            # user.last_name = 'Person'
            user.save()
            # Assert that the mock user was fetched and that the display name
            # was set.
            gmu.assert_called_once_with(user)
            mm_user.save.assert_called_once()
            self.assertEqual(mm_user.display_name, 'Anne')
            for addr in mm_user.addresses:
                self.assertEqual(addr.display_name, 'Anne')
                addr.save.assert_called_once()

    def test_create_profile_one_address_not_updated(self):
        # Similar to test_create_profile but tests that address.display_name
        # not updated if not changed.
        mm_user = Mock()
        mm_user.display_name = None
        # Mock user has two addresses.
        mm_user.addresses = [Mock(), Mock()]

        with patch('django_mailman3.signals.get_mailman_user') as gmu:
            gmu.return_value = mm_user
            user = User.objects.create_user(
                'myuser', 'testing@example.com', 'testPass')
            # Initially, the name of user is null, so we don't update the
            # display_name in Core.
            gmu.assert_not_called()
            # Now lets update the name of the user and see if it is reflected.
            user.first_name = 'Anne'
            user.last_name = 'Person'
            user.save()
            # Assert that the mock user was fetched and that the display name
            # was set.
            gmu.assert_called_once_with(user)
            mm_user.save.assert_called_once()
            self.assertEqual(mm_user.display_name, 'Anne Person')
            for addr in mm_user.addresses:
                self.assertEqual(addr.display_name, 'Anne Person')
                addr.save.assert_called_once()
            # Set second address display_name to new name.
            mm_user.addresses[1].display_name = 'Ann Person'
            # Reset the mocks and update the user.
            for addr in mm_user.addresses:
                addr.reset_mock()
            user.first_name = 'Ann'
            user.save()
            self.assertEqual(mm_user.display_name, 'Ann Person')
            self.assertEqual(mm_user.addresses[0].display_name, 'Ann Person')
            mm_user.addresses[0].save.assert_called_once()
            self.assertEqual(mm_user.addresses[1].display_name, 'Ann Person')
            mm_user.addresses[1].save.assert_not_called()



# Copyright (C) 2016-2023 by the Free Software Foundation, Inc.
#
# This file is part of Django-Mailman.
#
# Django-Mailman is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# Django-Mailman is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Django-Mailman.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import find_packages, setup


setup(
    name="django-mailman3",
    version="1.3.9",
    description="Django library to help interaction with Mailman",
    long_description=open('README.rst').read(),
    maintainer="Mailman Developers",
    maintainer_email="mailman-developers@python.org",
    license='GPLv3',
    keywords='mailman django',
    url="https://gitlab.com/mailman/django-mailman3",
    classifiers=[
        "Framework :: Django",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Topic :: Communications :: Email :: Mailing List Servers",
        "Programming Language :: Python :: 3",
        ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django>=3.2,<4.2',
        'mailmanclient>=3.3.3',
        'django-allauth',
        'django-gravatar2 >= 1.0.6',
        'pytz'
    ],
)



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        ('django_mailman3', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='MailDomain',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('mail_domain', models.CharField(unique=True, max_length=255, db_index=True)),
                ('site', models.ForeignKey(related_name='mailman_domains', to='sites.Site', on_delete=models.CASCADE)),
            ],
        ),
    ]
